/*************************************************************************
 *
 * FILE  transfer.cpp
 * Copyright (c) 2006, 2008 Daniel Kahlin <daniel@kahlin.net>
 * Written by Daniel Kahlin <daniel@kahlin.net>
 * $Id: transfer.cpp,v 1.33 2008-02-17 22:45:36 tlr Exp $
 *
 * DESCRIPTION 
 *
 ******/

#include <algorithm>
#include <stdio.h>
#include <cstring>
#include "global.hpp"

#include "transfer.hpp"

Transfer::Transfer(int blocksize, bool use_safe_setup_dtv)
    : blocksize_(blocksize), use_safe_setup_dtv_(use_safe_setup_dtv)
{
    clear_state_();
}

void Transfer::read_mem(uint8_t mode, uint32_t base, uint32_t len, uint8_t* target)
{
    set_timing_(use_safe_setup_dtv_);

    while (len) {
	uint16_t offs=static_cast<uint16_t>(base & 0x3fff);
	uint8_t  bank=static_cast<uint8_t>(base >> 14);
	uint16_t max_len=static_cast<uint16_t>(0x4000-offs);
	uint16_t block_len=std::min<int>(len, blocksize_);
	uint16_t tmp_len=std::min<int>(block_len, max_len);

	if (debug_g) {
	    printf("reading %s, bank=$%02X, offs=$%04X, len=$%04X (addr=$%06X)\n",(mode)?"ROM":"RAM",bank,offs,tmp_len,base);
	    fflush(stdout);
	}
	read_mem_raw(mode, bank, offs, tmp_len, target);
	base+=tmp_len;
	len-=tmp_len;
	target+=tmp_len;
    }
}

void Transfer::write_mem(uint8_t mode, uint32_t base, uint32_t len, uint8_t* source)
{
    get_revision_();

    while (len) {
	uint16_t offs=static_cast<uint16_t>(base & 0x3fff);
	uint8_t  bank=static_cast<uint8_t>(base >> 14);
	uint16_t max_len=static_cast<uint16_t>(0x4000-offs);
	uint16_t block_len=std::min<int>(len, blocksize_);
	uint16_t tmp_len=std::min<int>(block_len, max_len);

	if (debug_g) {
	    printf("writing %s, bank=$%02X, offs=$%04X, len=$%04X (addr=$%06X)\n",(mode)?"ROM":"RAM",bank,offs,tmp_len,base);
	    fflush(stdout);
	}
	write_mem_raw(mode, bank, offs, tmp_len, source);
	base+=tmp_len;
	len-=tmp_len;
	source+=tmp_len;
    }
}

void Transfer::read_mem(MemBuffer& mb)
{
    read_mem( (mb.is_rom())?1:0,  mb.get_start(), mb.get_length(), mb.get_buffer() );
}

uint16_t Transfer::read_word(uint32_t addr)
{
    uint8_t buf[2];
    read_mem(0,  addr, 2, buf );
    return buf[0] | (buf[1]<<8);
}

uint8_t Transfer::read_byte(uint32_t addr)
{
    uint8_t data;
    read_mem(0,  addr, 1, &data );
    return data;
}

void Transfer::write_mem(const MemBuffer& mb)
{
    write_mem( (mb.is_rom())?1:0,  mb.get_start(), mb.get_length(), mb.get_buffer() );
}

void Transfer::write_word(uint32_t addr, uint16_t data)
{
    uint8_t buf[2];
    buf[0]=data & 0xff;
    buf[1]=(data >> 8) & 0xff;
    write_mem(0, addr, 2, buf );
}

void Transfer::write_byte(uint32_t addr, uint8_t data)
{
    write_mem(0, addr, 1, &data );
}

void Transfer::execute_mem(uint16_t addr)
{
}

void Transfer::sys(uint16_t addr, uint8_t acc, uint8_t xr, uint8_t yr, uint8_t sr, uint8_t mem)
{
}

void Transfer::sys_result(uint8_t& acc, uint8_t& xr, uint8_t& yr, uint8_t& sr)
{
}


void Transfer::init(init_mode_t mode)
{
}

void Transfer::load(load_mode_t mode)
{
}

void Transfer::run(run_mode_t mode)
{
}

void Transfer::exit(exit_mode_t mode)
{
}

void Transfer::print(const char* str)
{
    int len = std::strlen(str);
    while (len > 0) {
	int n = std::min<int>(len, 256);
	print_raw(str, n);
	str += n;
	len -= n;
    }
}

void Transfer::print_raw(const char* str, int len)
{
}

void Transfer::color(uint8_t border, uint8_t background, uint8_t text)
{
}

bool Transfer::check_present(int timeout)
{
    return false;
}

void Transfer::tristate()
{
}

void Transfer::reset(bool knock)
{
}

void Transfer::send_file_boot(uint16_t start, uint16_t end, uint8_t* data)
{
}

const char* Transfer::get_implementation()
{
    return "<unimplemented>";
}

void Transfer::get_revision_()
{
    if (got_revision_)
	return;

    // check revision
    get_revision_raw();

    got_revision_ = true;

    if (debug_g) {
	printf("protocol revision: ");
	if (prev_major_<0)
	    printf("old\n");
	else
	    printf("%d.%d\n", prev_major_, prev_minor_);
    }
}

void Transfer::get_configuration_()
{
    if (got_configuration_)
	return;

    // check revision
    get_configuration_raw();

    got_configuration_ = true;
}

void Transfer::set_timing_(bool safe_setup_dtv)
{
    get_revision_();
    if (prev_major_ < 1)
	return;
    if (safe_setup_dtv_ == safe_setup_dtv)
	return;

    if (debug_g)
	printf("%s safe_setup_dtv\n", (safe_setup_dtv)?"enabling":"disabling");
    set_timing_raw(safe_setup_dtv);
    safe_setup_dtv_ = safe_setup_dtv;

    safe_timing_pc_ = !safe_setup_dtv;
}

void Transfer::clear_state_() 
{
    safe_timing_pc_ = true;
    safe_setup_dtv_ = false;
    got_revision_ = false;
    got_configuration_ = false;
}

void Transfer::read_mem_raw(uint8_t mode, uint8_t bank, uint16_t base, uint16_t len, uint8_t* target)
{
}

void Transfer::write_mem_raw(uint8_t mode, uint8_t bank, uint16_t base, uint16_t len, uint8_t* source)
{
}

void Transfer::get_revision_raw()
{
}

void Transfer::get_configuration_raw()
{
}

void Transfer::set_timing_raw(bool safe_setup_dtv)
{
}

/* eof */
